package org.knime.examples.credentials;

import javax.swing.JPanel;

import org.knime.core.data.DataTableSpec;
import org.knime.core.node.InvalidSettingsException;
import org.knime.core.node.NodeDialogPane;
import org.knime.core.node.NodeSettingsRO;
import org.knime.core.node.NodeSettingsWO;
import org.knime.core.node.NotConfigurableException;
import org.knime.core.node.defaultnodesettings.DefaultNodeSettingsPane;
import org.knime.core.node.defaultnodesettings.DialogComponentAuthentication;
import org.knime.core.node.defaultnodesettings.SettingsModelAuthentication;
import org.knime.core.node.defaultnodesettings.SettingsModelAuthentication.AuthenticationType;

/**
 * This is an example dialog that shows how to use the
 * {@link DialogComponentAuthentication}.
 * 
 * @author Moritz Heine, KNIME GmbH, Konstanz, Germany
 */
public class CredentialsExampleNodeDialog extends NodeDialogPane {

	/* Panel and model for authentication. */
	private final SettingsModelAuthentication m_authenticationModel;

	private final DialogComponentAuthentication m_authenticationPanel;

	protected CredentialsExampleNodeDialog() {
		m_authenticationModel = CredentialsExampleNodeModel.createAuthenticationModel();

		m_authenticationPanel = new DialogComponentAuthentication(m_authenticationModel, null,
				AuthenticationType.USER_PWD, AuthenticationType.CREDENTIALS);

		JPanel panel = new JPanel();

		panel.add(m_authenticationPanel.getComponentPanel());
		addTab("Options", panel);

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void saveSettingsTo(final NodeSettingsWO settings) throws InvalidSettingsException {
		m_authenticationModel.saveSettingsTo(settings);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void loadSettingsFrom(final NodeSettingsRO settings, final DataTableSpec[] specs)
			throws NotConfigurableException {
		try {
			m_authenticationModel.loadSettingsFrom(settings);

			m_authenticationPanel.loadSettingsFrom(settings, specs, getCredentialsProvider());
		} catch (InvalidSettingsException e) {
			throw new NotConfigurableException(e.getMessage(), e);
		}
	}
}
