package org.knime.examples.credentials;

import java.io.File;
import java.io.IOException;

import org.knime.core.data.DataColumnSpecCreator;
import org.knime.core.data.DataRow;
import org.knime.core.data.DataTableSpec;
import org.knime.core.data.def.DefaultRow;
import org.knime.core.data.def.StringCell;
import org.knime.core.node.BufferedDataContainer;
import org.knime.core.node.BufferedDataTable;
import org.knime.core.node.CanceledExecutionException;
import org.knime.core.node.ExecutionContext;
import org.knime.core.node.ExecutionMonitor;
import org.knime.core.node.InvalidSettingsException;
import org.knime.core.node.NodeModel;
import org.knime.core.node.NodeSettingsRO;
import org.knime.core.node.NodeSettingsWO;
import org.knime.core.node.defaultnodesettings.SettingsModelAuthentication;
import org.knime.core.node.defaultnodesettings.SettingsModelAuthentication.AuthenticationType;

/**
 * This is an example implementation of the node model of the
 * "CredentialsExampleNode".
 * 
 * @author Moritz Heine, KNIME GmbH, Konstanz, Germany
 */
public class CredentialsExampleNodeModel extends NodeModel {

	private final SettingsModelAuthentication m_authenticationModel;

	protected CredentialsExampleNodeModel() {
		super(0, 1);

		m_authenticationModel = CredentialsExampleNodeModel.createAuthenticationModel();
	}

	/**
	 * 
	 * {@inheritDoc}
	 */
	@Override
	protected BufferedDataTable[] execute(final BufferedDataTable[] inData, final ExecutionContext exec)
			throws Exception {
		DataTableSpec outputSpec = createOutputSpec();

		/*
		 * The execution context provides storage capacity, in this case a data
		 * container to which we will add rows sequentially. Note, this container can
		 * handle arbitrary big data tables, it will buffer to disc if necessary. The
		 * execution context is provided as an argument to the execute method by the
		 * framework. Have a look at the methods of the "exec". There is a lot of
		 * functionality to create and change data tables.
		 */
		BufferedDataContainer container = exec.createDataContainer(outputSpec);

		// Add the new row to the output data container
		// Note, that you have to provide the CredentialsProvider as this is used to access credentials.
		DataRow row = new DefaultRow("Row0", new StringCell(m_authenticationModel.getUserName(getCredentialsProvider())
				+ ":" + m_authenticationModel.getPassword(getCredentialsProvider())));
		container.addRowToTable(row);

		/*
		 * Once we are done, we close the container and return its table. Here we need
		 * to return as many tables as we specified in the constructor. This node has
		 * one output, hence return one table (wrapped in an array of tables).
		 */
		container.close();
		BufferedDataTable out = container.getTable();
		return new BufferedDataTable[] { out };
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected DataTableSpec[] configure(final DataTableSpec[] inSpecs) throws InvalidSettingsException {
		return new DataTableSpec[] { createOutputSpec() };
	}

	/**
	 * Creates the output table spec.
	 * 
	 * @return
	 */
	private DataTableSpec createOutputSpec() {
		return new DataTableSpec(new DataColumnSpecCreator("Credentials", StringCell.TYPE).createSpec());
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void loadInternals(final File nodeInternDir, final ExecutionMonitor exec)
			throws IOException, CanceledExecutionException {
		// TODO Auto-generated method stub

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void saveInternals(final File nodeInternDir, final ExecutionMonitor exec)
			throws IOException, CanceledExecutionException {
		// TODO Auto-generated method stub

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void saveSettingsTo(final NodeSettingsWO settings) {
		m_authenticationModel.saveSettingsTo(settings);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void validateSettings(final NodeSettingsRO settings) throws InvalidSettingsException {
		// TODO Auto-generated method stub

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void loadValidatedSettingsFrom(final NodeSettingsRO settings) throws InvalidSettingsException {
		m_authenticationModel.loadSettingsFrom(settings);
	}

	@Override
	protected void reset() {
		/*
		 * Code executed on a reset of the node. Models built during execute are cleared
		 * and the data handled in loadInternals/saveInternals will be erased.
		 */
	}

	/**
	 * @return Settings model for authentication.
	 */
	static final SettingsModelAuthentication createAuthenticationModel() {
		return new SettingsModelAuthentication("my-credentials", AuthenticationType.USER_PWD);
	}
}
